function [zT, zB] = pdZ(varargin)
% This function returns the transimpedance and backimpedance of the 
% aLIGO RFPD at the supplied frequency
%
%
% Ex.1   pdZ(logspace(6,8,333), godz(1,1:5), 9e6)

if nargin < 2
    error('Not enough args')
elseif nargin >1
    f = varargin{1};
    if nargin == 3
        x = varargin{2};
        w1 = 2*pi*varargin{3};
        L = zeros(size(x));
        R = zeros(size(x));
        C =  ones(size(x));
        % Convert impedance values into component values
        for m = 1:length(x)
            if imag(x(m)) > 0
                L(m) = imag(x(m))/w1;
            else
                C(m) = -1/w1/imag(x(m));
            end
            R(m) = real(x(m));
        end
    elseif nargin == 4
        R = varargin{2};
        L = varargin{3};
        C = varargin{4};
    end
end


%% InGaAs QPD parameters
Cd = 110e-12;
Rd = 23;
Ld = 5e-9;   % some kind of lead inductance of the photodiode

% Silicon QPD parameters
%Cd = 20e-12;
%Rd = 20;
f = f(:);
w = 2*pi*f;

%               Circuit Schematic
%     --------------------------------------
%     |         |        |                  |
%   diode      Z0       Z11                Z21
%               |        |    f1            |     f2
%              gnd       |-- opamp ->       |-- opamp ->
%                        |                  |
%                       Z12                Z22
%                        |                  |
%                       gnd                gnd

%% 

% Photodiode impedance seen from the output back in
zd = Rd + 1./(i*w*Cd) + i*w*Ld;

% Values for component impedances
z0  = R(1) + i*w*L(1) - i./w/C(1);
z11 = R(2) + i*w*L(2) - i./w/C(2);
z12 = R(3) + i*w*L(3) - i./w/C(3);
z21 = R(4) + i*w*L(4) - i./w/C(4);
z22 = R(5) + i*w*L(5) - i./w/C(5);


% Notch impedances
z1 = z11 + z12;
z2 = z21 + z22;

% Network impedance seen by the photodiode  (z0 || z1 || z2)
zNET = para(z1, z2);
zNET = para(zNET, z0);

% Total impedance seen by the diode current source (photocurrent)
zt = para(1./(i*w*Cd), (i*w*Ld + Rd + zNET));

% Readout impedance (volts out per amps of photocurrent)
zt2  = zt  .* (zNET ./ (i*w*Ld + Rd + zNET));
zr1  = zt2 .* (z12  ./ z1);   % at input of the  9 MHz readout amp
zr2  = zt2 .* (z22  ./ z2);   % at input of the 45 MHz readout amp


% impedance to ground from the readout node (the + input of the opamp)
zB1  = para(z12 , (z11 + para(z0, para(zd, z2)))); 
zB2  = para(z22 , (z21 + para(z0, para(zd, z1)))); 

% package impedances for the output
zT = [zr1(:) zr2(:)];
zB = [zB1(:) zB2(:)];


if nargout == 0
    subplot(211)
    semilogx(f, 20*log10(abs(zr1)), 'b',...
             f, 20*log10(abs(zr2)), 'r')
    grid
    ylabel('dB [Ohms]')
    axis([min(f) max(f) -20 80])
    lineAtX(9e6,'c')
    lineAtX(45e6,'m')

    
    subplot(212)
    semilogx(f, 180/pi*angle(zr1), 'bo',...
             f, 180/pi*angle(zr2), 'r')
    grid
    axis([min(f) max(f) -181 181])
    set(gca,'YTick',[-180:45:180])
    legend('f1 readout', 'f2 readout')
    xlabel('Hz')
    ylabel('Phase [deg]')
end



